<?php

namespace Litecms\Company\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litecms\Company\Forms\Master as MasterForm;
use Litecms\Company\Http\Requests\MasterResourceRequest;
use Litecms\Company\Http\Resources\MasterResource;
use Litecms\Company\Http\Resources\MastersCollection;
use Litecms\Company\Models\Master;
use Litecms\Company\Actions\MasterAction;
use Litecms\Company\Actions\MasterActions;


/**
 * Resource controller class for master.
 */
class MasterResourceController extends BaseController
{

    /**
     * Initialize master resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = MasterForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('litecms.company.modules'), 'company', guard_url('company'));
            return $next($request);
        });
    }

    /**
     * Display a list of master.
     *
     * @return Response
     */
    public function index(MasterResourceRequest $request)
    {
        $request = $request->all();
        $page = MasterActions::run('paginate', $request);

        $data = new MastersCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('company::master.names'))
            ->view('company::master.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display master.
     *
     * @param Request $request
     * @param Model   $master
     *
     * @return Response
     */
    public function show(MasterResourceRequest $request, Master $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new MasterResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('company::master.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('company::master.show')
            ->output();
    }

    /**
     * Show the form for creating a new master.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(MasterResourceRequest $request, Master $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new MasterResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('company::master.name'))
            ->view('company::master.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new master.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(MasterResourceRequest $request, Master $model)
    {
        try {
            $request = $request->all();
            $model = MasterAction::run('store', $model, $request);
            $data = new MasterResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('company::master.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('company/master/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/company/master'))
                ->redirect();
        }

    }

    /**
     * Show master for editing.
     *
     * @param Request $request
     * @param Model   $master
     *
     * @return Response
     */
    public function edit(MasterResourceRequest $request, Master $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new MasterResource($model);
        // return view('company::master.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('company::master.name'))
            ->view('company::master.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the master.
     *
     * @param Request $request
     * @param Model   $master
     *
     * @return Response
     */
    public function update(MasterResourceRequest $request, Master $model)
    {
        try {
            $request = $request->all();
            $model = MasterAction::run('update', $model, $request);
            $data = new MasterResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('company::master.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('company/master/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('company/master/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the master.
     *
     * @param Model   $master
     *
     * @return Response
     */
    public function destroy(MasterResourceRequest $request, Master $model)
    {
        try {

            $request = $request->all();
            $model = MasterAction::run('destroy', $model, $request);
            $data = new MasterResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('company::master.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('company/master/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('company/master/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
